<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Test\Constraint\Container\HasService;
use Inti\Test\Constraint\Container\ServiceInstanceOf;
use Laminas\ServiceManager\ServiceManager;
use PHPUnit\Framework\Assert;
use Psr\Container\ContainerInterface;

/**
 * Trait ContainerTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
trait ContainerTestCase
{
    use ConfigTestCase;

    private ?ContainerInterface $container = null;

    protected function setUpContainer(): void
    {
        $this->container = null;
    }

    /**
     * @param array $config
     * @return ContainerInterface
     */
    protected function createContainer(array $config = []): ContainerInterface
    {
        $config = $this->createConfig($config);

        $this->container = new ServiceManager($config['dependencies'] ?? []);
        $this->container->setService('config', $config);

        return $this->container;
    }

    /**
     * Provjerava da li je traženi servis registriran u containeru.
     *
     * @param string $service
     * @param string $message
     */
    public function assertContainerHasService(string $service, string $message = ''): void
    {
        if (!isset($this->container)) {
            $this->createContainer();
        }

        $hasService = new HasService($service);
        Assert::assertThat($this->container, $hasService, $message);
    }

    public function assertServiceInstanceOf(string $service, string $instanceOf, string $message = ''): void
    {
        $this->assertContainerHasService($service);

        $serviceInstanceOf = new ServiceInstanceOf($service, $instanceOf);
        Assert::assertThat($this->container, $serviceInstanceOf, $message);
    }
}
