<?php /** @noinspection PhpUndefinedMethodInspection */
/** @noinspection PhpParamsInspection */
/** @noinspection PhpFullyQualifiedNameUsageInspection */
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\Emitter\TestEmitter;
use Inti\Test\TestCase\ApplicationTestCase;
use Laminas\Diactoros\Response;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Laminas\Stratigility\Exception\EmptyPipelineException;
use Mezzio\Application;
use Mezzio\Container\ApplicationConfigInjectionDelegator;
use PHPUnit\Framework\TestCase;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;
use Psr\Http\Server\MiddlewareInterface;

/**
 * Class ApplicationTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ApplicationTest extends TestCase
{
    use ApplicationTestCase;
    use ProphecyTrait;

    protected array $providers = [
        \Mezzio\ConfigProvider::class,
        \Mezzio\Router\ConfigProvider::class,
        \Mezzio\Router\FastRouteRouter\ConfigProvider::class,
    ];

    protected array $config = [
        'dependencies' => [
            'delegators' => [
                Application::class => [
                    ApplicationConfigInjectionDelegator::class,
                ],
            ],
        ],
    ];

    public function testIsUsingTheTestEmitter()
    {
        $this->assertServiceInstanceOf(EmitterInterface::class, TestEmitter::class);
    }

    public function testCanCreateTheApplication()
    {
        $app = $this->createApplication();
        $this->assertInstanceOf(Application::class, $app);

        $this->expectException(EmptyPipelineException::class);
        $app->run();
    }

    public function testCanGetResponseFromApplication()
    {
        $expected = new Response();

        /** @var MiddlewareInterface|ObjectProphecy $middleware */
        $middleware = $this->prophesize(MiddlewareInterface::class);
        $middleware->process(Argument::any(), Argument::any())
            ->shouldBeCalled()
            ->willReturn($expected);

        $response = $this->runApplication([
            'middleware_pipeline' => [[
                'middleware' => $middleware->reveal(),
                'priority' => 0,
            ]],
        ]);

        $this->assertSame($expected, $response);
    }
}
