<?php
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\TestCase\ConfigTestCase;
use IntiTest\Test\Mock\MockConfigProvider;
use PHPUnit\Framework\ExpectationFailedException;
use PHPUnit\Framework\TestCase;

/**
 * Class ConfigTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ConfigTest extends TestCase
{
    use ConfigTestCase;

    private array $providers = [
        MockConfigProvider::class,
    ];

    private array $config = [
        'some_config' => [
            'some_key' => 'some value',
        ],
    ];

    public function testCanCheckIfKeyDoesntExists()
    {
        $config = $this->createConfig();

        $this->expectException(ExpectationFailedException::class);
        $this->assertConfigValueExists('dependencies/services/config', $config);
    }

    public function testCanCheckIfKeyExists()
    {
        $config = $this->createConfig();

        $this->assertConfigValueExists('some_config/some_key', $config);
    }

    public function testCanReadConfigFromProviders()
    {
        $config = $this->createConfig();

        $this->assertConfigValueExists('dependencies/aliases/mock_config_provider', $config);
        $this->assertEquals(MockConfigProvider::class, $config['dependencies']['aliases']['mock_config_provider']);
    }

    public function testWillUseDefaultConfig()
    {
        $config = $this->createConfig();

        $this->assertConfigValueExists('some_config/some_key', $config);
        $this->assertEquals('some value', $config['some_config']['some_key']);
    }

    public function testCanOverrideDefaultConfig()
    {
        $config = $this->createConfig([
            'some_config' => [
                'some_key' => 'other value',
            ],
        ]);

        $this->assertConfigValueExists('some_config/some_key', $config);
        $this->assertEquals('other value', $config['some_config']['some_key']);
    }
}
